// Steam Simulator, A.Greensted
// July 2010
// http://www.labbookpages.co.uk

package steamSim;

import steamSim.engines.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import java.net.*;

import org.w3c.dom.Document;
import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class EngineSelector extends JPanel implements ActionListener, ListSelectionListener, MouseListener
{
	private class EngineInfo
	{
		public String name;
		public Class engineClass;
		public ImageIcon icon;
		public String description;
		public String designer;
		public String simulationAuthor;
		public URL webLink;

		public String text;

		public String toString()
		{
			return name;
		}
	}

	private class EngineInfoRenderer implements ListCellRenderer, HyperlinkListener
	{
		private Color backgroundColour = new Color(240, 240, 240);

		private JPanel panel;
		private JLabel iconLabel;
		private JLabel textLabel;

		public EngineInfoRenderer()
		{
			panel = new JPanel(new BorderLayout(5, 5));

			panel.setBorder(new AbstractBorder()
			{
				public Insets getBorderInsets(Component c)
				{
					return new Insets(9,9,9,9);
				}
				public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
				{
					Color oldColour = g.getColor();

        			for (int i=0; i<5; i++)
					{
						if (i == 4) g.setColor(Color.BLACK);
						else g.setColor(Color.WHITE);
	             	g.drawRect(x+i, y+i, width-i-i-1, height-i-i-1);
					}

					g.setColor(oldColour);
				}
			});

			iconLabel = new JLabel();
			textLabel = new JLabel();

			Font font = textLabel.getFont().deriveFont(Font.PLAIN);
			textLabel.setFont(font);

			panel.add(iconLabel, BorderLayout.WEST);
			panel.add(textLabel, BorderLayout.CENTER);
		}

		public void hyperlinkUpdate(HyperlinkEvent e)
		{
			System.out.println(e);
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
		{
			EngineInfo info = (EngineInfo) value;

			if (info.icon != null) {
				iconLabel.setVisible(true);
				iconLabel.setIcon(info.icon);
			}
			else {
				iconLabel.setVisible(false);
				iconLabel.setIcon(null);
			}

			textLabel.setText(info.text);

			if (isSelected) {
				panel.setBackground(list.getSelectionBackground());
			}
			else {
				panel.setBackground(backgroundColour); //list.getBackground());
			}

			return panel;
		}
	}

	private EngineInfo[] engineArray;

	private JFrame owner;
	private JDialog dialog;
	private JList list;
	private JButton selectButton;
	private JButton cancelButton;

	private Class selectedEngineClass;

	public EngineSelector(JFrame owner)
	{
		this.owner = owner;

		loadXMLEngineList();

		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(new EmptyBorder(5,5,5,5));
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);

		list = new JList(engineArray);
		list.setVisibleRowCount(4);

		list.setCellRenderer(new EngineInfoRenderer());
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.addListSelectionListener(this);
		JScrollPane scrollPane = new JScrollPane(list);
		list.addMouseListener(this);

		selectButton = new JButton("Select");
		selectButton.addActionListener(this);
		selectButton.setEnabled(false);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(selectButton);
		buttonPanel.add(cancelButton);

		panel.add(scrollPane, BorderLayout.CENTER);
		panel.add(buttonPanel, BorderLayout.SOUTH);

		dialog = new JDialog(owner, "Engine Selector", true);
		dialog.getContentPane().add(panel);
		dialog.pack();
	}

	public void loadXMLEngineList()
	{
		ArrayList<EngineInfo> infoList = new ArrayList<EngineInfo>();

		try
		{
			InputStream stream = getClass().getResourceAsStream("/steamSim/engines/EngineList.xml");

			if (stream == null)
			{
				System.err.println("Failed to find list");
				return;
			}

			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(stream);

			doc.getDocumentElement().normalize();

			NodeList engineNodeList = doc.getElementsByTagName("engine");

			for (int n=0 ; n<engineNodeList.getLength() ; n++)
			{
				Node engineNode = engineNodeList.item(n);
				if (engineNode.getNodeType() != Node.ELEMENT_NODE) continue;

				Element engineElement = (Element) engineNode;

				EngineInfo engineInfo = new EngineInfo();

				// Get Engine Name
				engineInfo.name = getXMLValue(engineElement, "name");
				if (engineInfo.name == null) {
					System.err.println("EngineSelector: No Engine name");
					continue;
				}

				// Get Engine Class
				String className = getXMLValue(engineElement, "class");
				if (className == null) {
					System.err.println("EngineSelector: No Engine class name");
					continue;
				}

				engineInfo.engineClass = getEngineClass(className);
				if (engineInfo.engineClass == null) continue;

				// Engine Icon
				String iconName = getXMLValue(engineElement, "icon");
				if (iconName != null)
				{
					java.net.URL imgURL = getClass().getResource("/steamSim/engines/" + iconName);

					if (imgURL == null) {
						System.err.println("EngineSelector: Could not find engine icon '" + iconName + "'");
					}
					else {
						engineInfo.icon = new ImageIcon(imgURL);
					}
				}

				// Web Link
				String webLink = getXMLValue(engineElement, "webLink");
				if (webLink != null)
				{
					try
					{
						engineInfo.webLink = new URL(webLink);
					}
					catch (MalformedURLException e)
					{
						System.err.println("EngineSelector: Malformed URL '" + webLink + "'");
					}
				}

				// Other Parameters
				engineInfo.description = getXMLValue(engineElement, "description");
				engineInfo.designer = getXMLValue(engineElement, "designer");
				engineInfo.simulationAuthor = getXMLValue(engineElement, "simAuthor");

				engineInfo.text = "<html><p><b><font size=\"5\">" + engineInfo.name + "</font></b></p>";
				if (engineInfo.description != null) engineInfo.text += "<p>" + engineInfo.description + "</p>";
				if (engineInfo.designer != null) engineInfo.text += "<p><b>Designer:</b> " + engineInfo.designer + "</p>";
				if (engineInfo.simulationAuthor != null) engineInfo.text += "<p><b>Simulation Author:</b> " + engineInfo.simulationAuthor + "</p>";

//				if (engineInfo.webLink != null)
//					engineInfo.text += "<p><b>Further Details:</b> <a href=\"" + engineInfo.webLink + "\">" + engineInfo.webLink + "</a></p>";

				engineInfo.text += "</html>";

				infoList.add(engineInfo);
			}

			stream.close();
		}
		catch (Exception e)
		{
			System.err.println(e);
		}

		engineArray = new EngineInfo[infoList.size()];
		engineArray = infoList.toArray(engineArray);
	}

	public Class getEngineClass(String engineClassName)
	{
		Class engineClass = null;

		try
		{
			engineClass = Class.forName("steamSim.engines." + engineClassName);
		}
		catch (Exception e)
		{
			System.err.println("EngineSelector.getEngineClass: Could not find engine class '" + engineClassName + "'");
			return null;
		}

		if (!SteamEngine.class.isAssignableFrom(engineClass)) {
			System.out.println("EngineSelector.getEngineClass: Incorrect engine class type");
			return null;
		}

		return engineClass;
	}

	private String getXMLValue(Element engineElement, String name)
	{
		NodeList list = engineElement.getElementsByTagName(name);
		if (list.getLength() < 1) {
			return null;
		}

		Element element = (Element) list.item(0);
		NodeList elementList = element.getChildNodes();
		String value = ((Node)elementList.item(0)).getNodeValue().trim();

		if (value.length() == 0) return null;

		return value;
	}

	public void mouseClicked(MouseEvent e)
	{
		if (e.getClickCount() == 2) {
			dialog.setVisible(false);
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	public void valueChanged(ListSelectionEvent e)
	{
		if (e.getValueIsAdjusting() == false)
		{
			int index = list.getSelectedIndex();
			selectedEngineClass = engineArray[index].engineClass;
			selectButton.setEnabled(true);
		}
	}

	public void actionPerformed(ActionEvent aE)
	{
		Object o = aE.getSource();

		if (o == cancelButton) {
			selectedEngineClass = null;
			dialog.setVisible(false);
		}
		else if (o == selectButton) {
			dialog.setVisible(false);
		}
	}

	public Class showSelector()
	{
		dialog.setLocationRelativeTo(owner);
		dialog.setVisible(true);
		return selectedEngineClass;
	}
}
