// Steam Simulator, A.Greensted
// July 2010
// http://www.labbookpages.co.uk

package steamSim;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.lang.reflect.*;
import java.awt.print.*;

public abstract class SteamEngine extends JComponent implements MouseWheelListener, MouseInputListener, Serializable, Printable
{
	public final static Dimension PREFERRED_DIMENSION = new Dimension(600, 500);
	protected Parameter[] parametersArray;

	protected int prevX = 0;
	protected int prevY = 0;

	protected int transX = 0;
	protected int transY = 0;
	protected double scale = 1;

	protected int printTransX = 0;
	protected int printTransY = 0;
	protected double printScale = 1;

	public SteamEngine()
	{
		addMouseWheelListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);

		centreView();
	}

	public int print(Graphics g, PageFormat pf, int page) throws PrinterException
	{
		Graphics2D g2D = (Graphics2D)g;

		if (page == 0)
		{
			// Translate 0,0 to the corner of the printable area
			g2D.translate(pf.getImageableX(), pf.getImageableY());

			// Get font metrics for the current font
			FontMetrics metrics = g.getFontMetrics(g.getFont());

			// Find the maximum string width for Parameter names
			int maxWidth = 0;
			for (Parameter param : parametersArray) {
				int width = metrics.stringWidth(param.name);
				if (width > maxWidth) maxWidth = width;
			}

			// Print out parameter names and values
			int yPos = 15;
			int valOffset = maxWidth + 20;

			for (Parameter param : parametersArray)
			{
				g.drawString(param.name, 0, yPos);
				g.drawString(Double.toString(param.value), valOffset, yPos);
				yPos += 15;
			}

			return PAGE_EXISTS;
		}

		if (page == 1)
		{
			// Translate 0,0 to centre of the printable area
			g2D.translate(	(int)(pf.getImageableX() + pf.getImageableWidth()/2.0),
								(int)(pf.getImageableY() + pf.getImageableHeight()/2.0));

			paintEngine(g2D, true, printTransX, printTransY, printScale);
			return PAGE_EXISTS;
		}

		return NO_SUCH_PAGE;
	}

	public void paintComponent(Graphics g)
	{
   	super.paintComponent(g);

		Graphics2D g2D = (Graphics2D) g;
		g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		Insets insets = getInsets();

		// Get size of available area
		int width = getWidth() - insets.left - insets.right;
		int height = getHeight() - insets.top - insets.bottom;

		// Background and border
		g.setColor(Color.BLACK);
		g.fillRect(insets.left, insets.top, width-1, height-1);
		g.setColor(Color.GRAY);
		g.drawRect(insets.left, insets.top, width-1, height-1);

		// Set clip so drawing does not go outside area when zoomed in
		g.setClip(insets.left+1, insets.top+1, width-2, height-2);

		// Calculate centre point of drawing space
		int centreX = insets.left + width / 2;
		int centreY = insets.top + height / 2;

		g2D.translate(centreX, centreY);
		paintEngine(g2D, false, transX, transY, scale);

		// Reset Translation & Clip
		g2D.translate(-centreX, -centreY);
		g2D.setClip(0,0,getWidth(), getHeight());
	}

	public abstract void paintEngine(Graphics2D g2D, boolean printjob, int tX, int tY, double s);

	public void printOut()
	{
		System.out.println(this.getClass());
		Parameter[] array = getParametersArray();
		for (Parameter param : array) {
			System.out.printf(" - %s: %f\n", param.name, param.value);
		}
	}

	public void centreView()
	{
		transX = 0;
		transY = 0;
		scale = 8.0;
	}

	public abstract void resetAnimation();
	public abstract void stepAnimationBack();
	public abstract void stepAnimationForward();

	public Parameter[] getParametersArray()
	{
		return parametersArray;
	}

	public Dimension getSize()
	{
		return super.getSize();
	}

	public Dimension getPreferredSize()
	{
		Insets insets = getInsets();
		return new Dimension(PREFERRED_DIMENSION.width + insets.left + insets.right,
									PREFERRED_DIMENSION.height + insets.top + insets.bottom);
	}

	protected void drawCircle(Graphics2D g, int tX, int tY, double s, double x, double y, double radius)
	{
		int xi = tX + (int)((x-radius) * s);
		int yi = tY + (int)((y-radius) * s);
		int size = (int)(2.0 * radius * s);

		g.drawOval(xi, yi, size, size);
	}

	protected void drawCircle(Graphics2D g,	int tX, int tY, double s,
															double rotPntX, double rotPntY, double angle,
															double x, double y, double radius)
	{
		double cos = Math.cos(angle);
		double sin = Math.sin(angle);

		double dx = x - rotPntX;
		double dy = y - rotPntY;

		double xr = rotPntX + (cos * dx - sin * dy);
		double yr = rotPntY + (sin * dx + cos * dy);

		int xi = tX + (int)((xr-radius) * s);
		int yi = tY + (int)((yr-radius) * s);
		int size = (int)(2.0 * radius * s);

		g.drawOval(xi, yi, size, size);
	}

	protected void drawLine(Graphics2D g, int tX, int tY, double s, double x1, double y1, double x2, double y2)
	{
		int x1i = tX + (int)(x1 * s);
		int y1i = tY + (int)(y1 * s);
		int x2i = tX + (int)(x2 * s);
		int y2i = tY + (int)(y2 * s);

		g.drawLine(x1i,y1i,x2i,y2i);
	}

	protected void drawRect(Graphics2D g, int tX, int tY, double s, double x1, double y1, double x2, double y2)
	{
		int x1i = tX + (int)(x1 * s);
		int y1i = tY + (int)(y1 * s);
		int x2i = tX + (int)(x2 * s);
		int y2i = tY + (int)(y2 * s);

		g.drawLine(x1i,y1i,x2i,y1i);
		g.drawLine(x2i,y1i,x2i,y2i);
		g.drawLine(x2i,y2i,x1i,y2i);
		g.drawLine(x1i,y2i,x1i,y1i);
	}

	protected void drawLine(Graphics2D g,	int tX, int tY, double s,
														double rotPntX, double rotPntY, double angle,
														double x1, double y1, double x2, double y2)
	{
		double cos = Math.cos(angle);
		double sin = Math.sin(angle);

		double dx1 = x1 - rotPntX;
		double dx2 = x2 - rotPntX;
		double dy1 = y1 - rotPntY;
		double dy2 = y2 - rotPntY;

		double x1r = rotPntX + (cos * dx1 - sin * dy1);
		double y1r = rotPntY + (sin * dx1 + cos * dy1);
                                          
		double x2r = rotPntX + (cos * dx2 - sin * dy1);
		double y2r = rotPntY + (sin * dx2 + cos * dy1);

		int x1i = tX + (int)(x1r * s);
		int y1i = tY + (int)(y1r * s);

		int x2i = tX + (int)(x2r * s);
		int y2i = tY + (int)(y2r * s);

		g.drawLine(x1i,y1i,x2i,y2i);
	}

	protected void drawRect(Graphics2D g,	int tX, int tY, double s,
														double rotPntX, double rotPntY, double angle,
														double x1, double y1, double x2, double y2)
	{
		double cos = Math.cos(angle);
		double sin = Math.sin(angle);

		double dx1 = x1 - rotPntX;
		double dx2 = x2 - rotPntX;
		double dy1 = y1 - rotPntY;
		double dy2 = y2 - rotPntY;

		double x1r = rotPntX + (cos * dx1 - sin * dy1);
		double y1r = rotPntY + (sin * dx1 + cos * dy1);
                                          
		double x2r = rotPntX + (cos * dx2 - sin * dy1);
		double y2r = rotPntY + (sin * dx2 + cos * dy1);
                                          
		double x3r = rotPntX + (cos * dx2 - sin * dy2);
		double y3r = rotPntY + (sin * dx2 + cos * dy2);
                                          
		double x4r = rotPntX + (cos * dx1 - sin * dy2);
		double y4r = rotPntY + (sin * dx1 + cos * dy2);

		int x1i = tX + (int)(x1r * s);
		int y1i = tY + (int)(y1r * s);

		int x2i = tX + (int)(x2r * s);
		int y2i = tY + (int)(y2r * s);

		int x3i = tX + (int)(x3r * s);
		int y3i = tY + (int)(y3r * s);

		int x4i = tX + (int)(x4r * s);
		int y4i = tY + (int)(y4r * s);

		g.drawLine(x1i,y1i,x2i,y2i);
		g.drawLine(x2i,y2i,x3i,y3i);
		g.drawLine(x3i,y3i,x4i,y4i);
		g.drawLine(x4i,y4i,x1i,y1i);
	}

	public void mouseWheelMoved(MouseWheelEvent e)
	{
		int scrollAmount = e.getWheelRotation();

		double factor = ((e.getModifiersEx() & MouseWheelEvent.CTRL_DOWN_MASK) == 0) ? -0.5 : -0.1;
		scale += scrollAmount * factor;
		if (scale < 0.5) scale = 0.5;

		repaint();
	}

	public void mousePressed(MouseEvent e)
	{
		prevX = e.getX();
		prevY = e.getY();
	}

	public void mouseDragged(MouseEvent e)
	{
		int dX = e.getX() - prevX;
		int dY = e.getY() - prevY;

		transX += dX;
		transY += dY;

		prevX = e.getX();
		prevY = e.getY();

		repaint();
	}

	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {}

	public static SteamEngine readFromStream(InputStream inputStream)
	{
		DataInputStream stream = null;
		Class engineClass = null;
		SteamEngine engine = null;

		try
		{
			stream = new DataInputStream(inputStream);

			String engineClassName = stream.readUTF();
			ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
			
			System.out.println(engineClassName);
			engineClass = classLoader.loadClass(engineClassName);
			engine = (SteamEngine) engineClass.newInstance();
		}
		catch (Exception e)
		{
			System.err.println("SteamEngine.readFromStream");
			System.err.println(e);
		}

		if (engine != null)
		{
			try
			{
				while (true)
				{
					String fieldName = stream.readUTF();
					double val = stream.readDouble();
					
					try
					{
						Field field = engineClass.getField(fieldName);
						Parameter param = (Parameter) field.get(engine);
						param.value = val;
					}
					catch (NoSuchFieldException e)
					{
						System.err.println("Field '" + fieldName + "' not found");
						continue;
					}
				}
			}	
			catch (EOFException e)
			{
			}
			catch (Exception e)
			{
				System.err.println("SteamEngine.readFromStream");
				System.err.println(e);
			}
		}

		try
		{
			stream.close();
		}
		catch (Exception e)
		{
			System.err.println("SteamEngine.readFromStream close");
			System.err.println(e);
		}

		return engine;
	}

	public static void writeToStream(OutputStream outputStream, SteamEngine engine)
	{
		try
		{
			DataOutputStream stream = new DataOutputStream(outputStream);

			Class engineClass = engine.getClass();
			String engineClassName = engineClass.getName();
			System.out.println(engineClassName);
			stream.writeUTF(engineClassName);

			Field[] fields = engineClass.getFields();

			for (Field field : fields)
			{
				if (field.getType() != Parameter.class) continue;
				
				String name = field.getName();
				Parameter param = (Parameter) field.get(engine);
				double val = param.value;

				stream.writeUTF(name);
				stream.writeDouble(val);
			}

			stream.close();
		}
		catch (Exception e)
		{
			System.err.println(e);
		}
	}
}
