// Steam Simulator, A.Greensted
// July 2010
// http://www.labbookpages.co.uk

package steamSim.engines;

import steamSim.*;
import java.awt.*;
import javax.swing.*;

public class DoubleOscillator extends SteamEngine
{
	private final static Color GUIDE_COLOUR = new Color(40,40,40);

	// Dimensions all in mm
	public Parameter axleSpacing			= new Parameter(	35.0, "Axle Spacing",
																			"The distance between the crank axle and the cylinder axle", Color.YELLOW);

	public Parameter crankLength			= new Parameter(	8.0, "Crank Length",
																			"The distance between the crank axle and the crank pin", Color.RED);

	public Parameter pistonRodLength		= new Parameter(	35.0, "Piston Rod Length",
																			"Distance between crank pin and centre of piston", Color.BLUE);

	public Parameter pistonLength			= new Parameter(	4.0, "Piston Length",
																			"The length of the piston head", Color.CYAN);
	public Parameter pistonRadius			= new Parameter(	6.0, "Piston Radius",
																			"The radius of the piston head", Color.CYAN);

	public Parameter cylinderLength		= new Parameter(	25.0, "Cylinder Length",
																			"Length of cylinder", Color.PINK);
	public Parameter cylinderPortRadius	= new Parameter(	1.0, "Cylinder Port Radius",
																			"Radius of the cylinder port", Color.PINK);
	public Parameter cylinderPortOffset	= new Parameter(	10.0, "Cylinder Port Offset",
																			"Distance between cylinder axle and port", Color.PINK);

	public Parameter facePortRadius		= new Parameter(	1.0, "Face Port Radius",
																			"Radius of port hole in port face", Color.GREEN);
	public Parameter facePortOffsetX		= new Parameter(	10.0, "Face Port Offset X",
																			"X offset from cylinder axle for face port", Color.GREEN);
	public Parameter facePortOffsetY		= new Parameter(	2.5, "Face Port Offset Y",
																			"Y offset from cylinder axle for face port", Color.GREEN);

	public Parameter cylinderAxleRadius	= new Parameter(	2.0, "Cylinder Axle Radius",
																			"Radius of the cylinder axle", Color.ORANGE);
	public Parameter crankAxleRadius		= new Parameter(	3.0, "Crank Axle Radius",
																			"Radius of the crank axle", Color.ORANGE);
	public Parameter crankPinRadius		= new Parameter(	1.5, "Crank Pin Radius",
																			"Radius of the crank pin", Color.ORANGE);
	public Parameter crankPlateRadius	= new Parameter(	12.0, "Crank Plate Radius",
																			"Radius of the crank Plate", Color.ORANGE);


	private double crankAngle = 0;

	public DoubleOscillator()
	{
		parametersArray = new Parameter[] {
			axleSpacing, crankLength, pistonRodLength, pistonLength,	pistonRadius,
			cylinderLength, cylinderPortRadius, cylinderPortOffset,
			facePortRadius, facePortOffsetX, facePortOffsetY,
			cylinderAxleRadius, crankAxleRadius, crankPinRadius, crankPlateRadius,
		};

		printScale = 6.0;
	}

	public void resetAnimation()
	{
		crankAngle = 0;
	}

	public void stepAnimationBack()
	{
		crankAngle -= Math.PI / 60;
		if (crankAngle < 0) crankAngle += 2 * Math.PI;
	}

	public void stepAnimationForward()
	{
		crankAngle += Math.PI / 60;
		if (crankAngle > 2 * Math.PI) crankAngle -= 2 * Math.PI;
	}

	public void paintEngine(Graphics2D g2D, boolean printjob, int tX, int tY, double s)
	{
		// Calculate position of Cylinder Axle, Crank Axle and Crank Pin
		double cylinderAxleX = axleSpacing.value / 2;
		double crankAxleX = -axleSpacing.value / 2;
		double crankPinX = crankAxleX + crankLength.value * Math.cos(crankAngle);
		double crankPinY = crankLength.value * Math.sin(crankAngle);

		// Calculate the distance between the Cylinder Axle and the Crank Pin
		double distA = Math.sqrt(	axleSpacing.value * axleSpacing.value +
											crankLength.value * crankLength.value -
											2 * axleSpacing.value * crankLength.value * Math.cos(crankAngle));
		double cylinderAngle = Math.asin(Math.sin(crankAngle) * crankLength.value / distA);

		// Calculate Piston Path (line between Crank Pin and Cylinder Axle)
		double pistonPathXS = axleSpacing.value / 2 - (distA + crankLength.value) * Math.cos(cylinderAngle);
		double pistonPathYS = (distA + crankLength.value) * Math.sin(cylinderAngle);
		double pistonPathXE = axleSpacing.value / 2 + (0.75 * cylinderLength.value) * Math.cos(cylinderAngle);
		double pistonPathYE = -0.75 * cylinderLength.value * Math.sin(cylinderAngle);

		// Calculate Centre point of piston
		double pistonCentreX = crankPinX + pistonRodLength.value * Math.cos(cylinderAngle);
		double pistonCentreY = crankPinY - pistonRodLength.value * Math.sin(cylinderAngle);

		// Calculate Cylinder Walls
		double cy1X = Math.cos(-cylinderAngle) * (-cylinderLength.value/2) - Math.sin(-cylinderAngle) * pistonRadius.value;
		double cy1Y = Math.sin(-cylinderAngle) * (-cylinderLength.value/2) + Math.cos(-cylinderAngle) * pistonRadius.value;
		double cy2X = Math.cos(-cylinderAngle) * (cylinderLength.value/2) - Math.sin(-cylinderAngle) * pistonRadius.value;
		double cy2Y = Math.sin(-cylinderAngle) * (cylinderLength.value/2) + Math.cos(-cylinderAngle) * pistonRadius.value;

		double cyPortPosX = cylinderPortOffset.value * Math.cos(cylinderAngle);
		double cyPortPosY = cylinderPortOffset.value * Math.sin(cylinderAngle);

		// Calculate Piston Walls
		double pi1X = Math.cos(-cylinderAngle) * (-pistonLength.value/2) - Math.sin(-cylinderAngle) * pistonRadius.value;
		double pi1Y = Math.sin(-cylinderAngle) * (-pistonLength.value/2) + Math.cos(-cylinderAngle) * pistonRadius.value;
		double pi2X = Math.cos(-cylinderAngle) * (pistonLength.value/2) - Math.sin(-cylinderAngle) * pistonRadius.value;
		double pi2Y = Math.sin(-cylinderAngle) * (pistonLength.value/2) + Math.cos(-cylinderAngle) * pistonRadius.value;

		// Guide Lines
		g2D.setColor(GUIDE_COLOUR);
		drawLine(g2D, tX, tY, s, -axleSpacing.value, 0, axleSpacing.value, 0);														// Horizontal Centre Line
		drawLine(g2D, tX, tY, s, crankAxleX, crankLength.value * 1.5, crankAxleX, - crankLength.value * 1.5);				// Vertical Crank Axle Line
		drawLine(g2D, tX, tY, s, cylinderAxleX, crankLength.value * 1.5, cylinderAxleX, - crankLength.value * 1.5);		// Vertical Cylinder Axle Line
		drawLine(g2D, tX, tY, s, pistonPathXS, pistonPathYS, pistonPathXE, pistonPathYE);										// Piston Path

		// Axle Spacing
		g2D.setColor(axleSpacing.colour);
		drawLine(g2D, tX, tY, s, crankAxleX, 0, cylinderAxleX, 0);

		// Face Ports
		g2D.setColor(facePortRadius.colour);
		drawCircle(g2D, tX, tY, s, cylinderAxleX-facePortOffsetX.value, facePortOffsetY.value, facePortRadius.value);
		drawCircle(g2D, tX, tY, s, cylinderAxleX-facePortOffsetX.value, -facePortOffsetY.value, facePortRadius.value);
		drawCircle(g2D, tX, tY, s, cylinderAxleX+facePortOffsetX.value, facePortOffsetY.value, facePortRadius.value);
		drawCircle(g2D, tX, tY, s, cylinderAxleX+facePortOffsetX.value, -facePortOffsetY.value, facePortRadius.value);

		// Cylinder Ports
		g2D.setColor(cylinderLength.colour);
		drawCircle(g2D, tX, tY, s, cylinderAxleX-cyPortPosX, cyPortPosY, cylinderPortRadius.value);		// Port nearest to crank
		drawCircle(g2D, tX, tY, s, cylinderAxleX+cyPortPosX, -cyPortPosY, cylinderPortRadius.value);		// Port furthest from crank

		// Crank
		g2D.setColor(crankLength.colour);
		drawLine(g2D, tX, tY, s, crankAxleX, 0, crankPinX, crankPinY);

		// Axles, Crank Pin and Crank Plate
		g2D.setColor(cylinderAxleRadius.colour);
		drawCircle(g2D, tX, tY, s, cylinderAxleX, 0, cylinderAxleRadius.value);
		g2D.setColor(crankPinRadius.colour);
		drawCircle(g2D, tX, tY, s, crankPinX, crankPinY, crankPinRadius.value);
		g2D.setColor(crankAxleRadius.colour);
		drawCircle(g2D, tX, tY, s, crankAxleX, 0, crankAxleRadius.value);
		g2D.setColor(crankPlateRadius.colour);
		drawCircle(g2D, tX, tY, s, crankAxleX, 0, crankPlateRadius.value);

		// Piston
		g2D.setColor(pistonRodLength.colour);
		drawLine(g2D, tX, tY, s, crankPinX, crankPinY, pistonCentreX, pistonCentreY);								// Piston Rod

		g2D.setColor(pistonRadius.colour);
		drawLine(g2D, tX, tY, s,	pistonCentreX + pi1X, pistonCentreY + pi1Y,
											pistonCentreX - pi2X, pistonCentreY - pi2Y);							// Near Face
		drawLine(g2D, tX, tY, s,	pistonCentreX - pi1X, pistonCentreY - pi1Y,
											pistonCentreX + pi2X, pistonCentreY + pi2Y);							// Far Face

		// Cylinder Walls
		g2D.setColor(cylinderLength.colour);
		drawLine(g2D, tX, tY, s, cylinderAxleX + cy1X,  cy1Y, cylinderAxleX + cy2X,  cy2Y);
		drawLine(g2D, tX, tY, s, cylinderAxleX - cy1X, -cy1Y, cylinderAxleX - cy2X, -cy2Y);
		drawLine(g2D, tX, tY, s, cylinderAxleX + cy1X,  cy1Y, cylinderAxleX - cy2X, -cy2Y);						// Near Face
		drawLine(g2D, tX, tY, s, cylinderAxleX - cy1X, -cy1Y, cylinderAxleX + cy2X,  cy2Y);						// Far Face
	}
}
