// Steam Simulator, A.Greensted
// July 2010
// http://www.labbookpages.co.uk

package steamSim.engines;

import steamSim.*;
import java.awt.*;
import javax.swing.*;

public class FullDoubleOscillator extends SteamEngine
{
	private final static Color GUIDE_COLOUR = new Color(40,40,40);

	// Dimensions all in mm
	public Parameter crankAxlePosition	= new Parameter(	15.0, "Crank Axle Position",
																			"The distance from left hand block edge to crank axle centre", Color.ORANGE);

	public Parameter crankLength			= new Parameter(	9.0, "Crank Length",
																			"The distance between the crank axle and the crank pin", Color.RED);

	public Parameter crankAxleRadius		= new Parameter(	3.0, "Crank Axle Radius",
																			"Radius of the crank axle", Color.ORANGE);
	public Parameter crankPinRadius		= new Parameter(	1.5, "Crank Pin Radius",
																			"Radius of the crank pin", Color.ORANGE);
	public Parameter crankPlateRadius	= new Parameter(	12.5, "Crank Plate Radius",
																			"Radius of the crank Plate", Color.ORANGE);

	public Parameter pistonRodLength		= new Parameter(	40.0, "Piston Rod Length",
																			"Distance between crank pin and centre of piston", Color.BLUE);

	public Parameter pistonLength			= new Parameter(	4.0, "Piston Length",
																			"The length of the piston head", Color.CYAN);
	public Parameter pistonRadius			= new Parameter(	6.0, "Piston Radius",
																			"The radius of the piston head", Color.CYAN);

	public Parameter cylinderAxleRadius	= new Parameter(	2.0, "Cylinder Axle Radius",
																			"Radius of the cylinder axle", Color.ORANGE);

	public Parameter cylinderAxlePosition	= new Parameter(	55.0, "Cylinder Axle Position",
																			"The distance from left hand block edge to cylinder axle centre", Color.PINK);

	public Parameter cylinderRadius		= new Parameter(	12.5, "Cylinder Radius",
																			"Radius of Cylinder", Color.PINK);
	public Parameter cylinderLength		= new Parameter(	30.0, "Cylinder Length",
																			"Length of cylinder", Color.PINK);
	public Parameter cylinderPortRadius	= new Parameter(	1.0, "Cylinder Port Radius",
																			"Radius of the cylinder port", Color.PINK);
	public Parameter cylinderPortOffset	= new Parameter(	11.0, "Cylinder Port Offset",
																			"Distance between cylinder axle and port", Color.PINK);

	public Parameter cylinderCapWidth	= new Parameter(	3.0, "Cylinder Cap Width",
																			"Width of cylinder cap", Color.MAGENTA);
	public Parameter cylinderCapRadius	= new Parameter(	10.0, "Cylinder Cap Radius",
																			"Radius of cylinder cap", Color.MAGENTA);
	public Parameter glandRadius			= new Parameter(	5.0, "Cylinder Cap Gland Radius",
																			"Radius of cylinder cap gland", Color.MAGENTA);
	public Parameter glandLength			= new Parameter(	3.0, "Cylinder Cap Gland Length",
																			"Length of cylinder cap gland", Color.MAGENTA);

	public Parameter blockLength			= new Parameter(	75.0, "Block Length",
																			"Length of main block", Color.LIGHT_GRAY);
	public Parameter blockHeight			= new Parameter(	20.0, "Block Height",
																			"Height of main block", Color.LIGHT_GRAY);

	public Parameter facePortRadius		= new Parameter(	1.0, "Face Port Radius",
																			"Radius of port hole in port face", Color.GREEN);
	public Parameter facePortOffsetX		= new Parameter(	11.0, "Face Port Offset X",
																			"X offset from cylinder axle for face port", Color.GREEN);
	public Parameter facePortOffsetY		= new Parameter(	2.5, "Face Port Offset Y",
																			"Y offset from cylinder axle for face port", Color.GREEN);

	public Parameter flyWheelRadius		= new Parameter(	25.0, "Fly Wheel Radius",
																			"Radius of the fly wheel, connected to crank axle", Color.YELLOW);

	public Parameter bigEndLength			= new Parameter(	10.0, "Big End Length",
																			"Length of big end", Color.CYAN);

	public Parameter bigEndWidth			= new Parameter(	6.0, "Big End Width",
																			"Width of big end", Color.CYAN);

	public Parameter bigEndHoleOffset	= new Parameter(	5.0, "Big End Hole Offset",
																			"Position of crank pin hole, taken from piston rod face", Color.CYAN);


	private double crankAngle = 0;

	public FullDoubleOscillator()
	{
		parametersArray = new Parameter[] {
			crankAxlePosition, crankLength,
			crankAxleRadius, crankPinRadius, crankPlateRadius,
			pistonRodLength, pistonLength, pistonRadius,
			cylinderAxleRadius, 
			cylinderAxlePosition, cylinderRadius, cylinderLength, cylinderPortRadius, cylinderPortOffset,
			cylinderCapWidth, cylinderCapRadius,
			glandRadius, glandLength,
			blockLength, blockHeight,
			facePortRadius, facePortOffsetX, facePortOffsetY,
			flyWheelRadius,
			bigEndLength, bigEndWidth, bigEndHoleOffset,
		};

		printScale = 5.0;
		printTransX = 30;
	}

	public void centreView()
	{
		transX = 30;
		transY = 0;
		scale = 6.0;
	}

	public void resetAnimation()
	{
		crankAngle = 0;
	}

	public void stepAnimationBack()
	{
		crankAngle -= Math.PI / 60;
		if (crankAngle < 0) crankAngle += 2 * Math.PI;
	}

	public void stepAnimationForward()
	{
		crankAngle += Math.PI / 60;
		if (crankAngle > 2 * Math.PI) crankAngle -= 2 * Math.PI;
	}

	public void paintEngine(Graphics2D g2D, boolean printjob, int tX, int tY, double s)
	{
		// Calculate position of Cylinder Axle, Crank Axle and Crank Pin
		double cylinderAxleX = cylinderAxlePosition.value - (blockLength.value / 2);
		double crankAxleX = crankAxlePosition.value - (blockLength.value / 2);
		double crankPinX = crankAxleX + crankLength.value * Math.cos(crankAngle);
		double crankPinY = crankLength.value * Math.sin(crankAngle);

		double axleSpacing = cylinderAxlePosition.value - crankAxlePosition.value;

		// Calculate the distance between the Cylinder Axle and the Crank Pin
		double distA = Math.sqrt(	axleSpacing * axleSpacing +
											crankLength.value * crankLength.value -
											2 * axleSpacing * crankLength.value * Math.cos(crankAngle));
		double cylinderAngle = Math.asin(Math.sin(crankAngle) * crankLength.value / distA);

		// Calculate Centre point of piston
		double pistonCentreX = crankPinX + pistonRodLength.value * Math.cos(cylinderAngle);
		double pistonCentreY = crankPinY - pistonRodLength.value * Math.sin(cylinderAngle);

		double cyPortPosX = cylinderPortOffset.value * Math.cos(cylinderAngle);
		double cyPortPosY = cylinderPortOffset.value * Math.sin(cylinderAngle);

		// Calculate Piston Walls
		double pi1X = Math.cos(-cylinderAngle) * (-pistonLength.value/2) - Math.sin(-cylinderAngle) * pistonRadius.value;
		double pi1Y = Math.sin(-cylinderAngle) * (-pistonLength.value/2) + Math.cos(-cylinderAngle) * pistonRadius.value;
		double pi2X = Math.cos(-cylinderAngle) * (pistonLength.value/2) - Math.sin(-cylinderAngle) * pistonRadius.value;
		double pi2Y = Math.sin(-cylinderAngle) * (pistonLength.value/2) + Math.cos(-cylinderAngle) * pistonRadius.value;

		// Guide Lines
		g2D.setColor(GUIDE_COLOUR);
		drawLine(g2D, tX, tY, s, -0.6*blockLength.value, 0, 0.6*blockLength.value, 0);											// Horizontal Centre Line
		drawLine(g2D, tX, tY, s, crankAxleX, crankLength.value * 1.5, crankAxleX, - crankLength.value * 1.5);				// Vertical Crank Axle Line
		drawLine(g2D, tX, tY, s, cylinderAxleX, crankLength.value * 1.5, cylinderAxleX, - crankLength.value * 1.5);		// Vertical Cylinder Axle Line

		g2D.setColor(blockLength.colour);
		drawRect(g2D, tX, tY, s, -(blockLength.value/2), (blockHeight.value/2),
													(blockLength.value/2), -(blockHeight.value/2));

		// Fly Wheel											
		g2D.setColor(flyWheelRadius.colour);
		drawCircle(g2D, tX, tY, s, crankAxleX, 0, flyWheelRadius.value);

		// Face Ports
		g2D.setColor(facePortRadius.colour);
		drawCircle(g2D, tX, tY, s, cylinderAxleX-facePortOffsetX.value, facePortOffsetY.value, facePortRadius.value);
		drawCircle(g2D, tX, tY, s, cylinderAxleX-facePortOffsetX.value, -facePortOffsetY.value, facePortRadius.value);
		drawCircle(g2D, tX, tY, s, cylinderAxleX+facePortOffsetX.value, facePortOffsetY.value, facePortRadius.value);
		drawCircle(g2D, tX, tY, s, cylinderAxleX+facePortOffsetX.value, -facePortOffsetY.value, facePortRadius.value);

		// Cylinder Ports
		g2D.setColor(cylinderLength.colour);
		drawCircle(g2D, tX, tY, s, cylinderAxleX-cyPortPosX, cyPortPosY, cylinderPortRadius.value);				// Port nearest to crank
		drawCircle(g2D, tX, tY, s, cylinderAxleX+cyPortPosX, -cyPortPosY, cylinderPortRadius.value);				// Port furthest from crank

		// Crank
		g2D.setColor(crankLength.colour);
		drawLine(g2D, tX, tY, s, crankAxleX, 0, crankPinX, crankPinY);

		// Axles, Crank Pin and Crank Plate
		g2D.setColor(cylinderAxleRadius.colour);
		drawCircle(g2D, tX, tY, s, cylinderAxleX, 0, cylinderAxleRadius.value);
		g2D.setColor(crankPinRadius.colour);
		drawCircle(g2D, tX, tY, s, crankPinX, crankPinY, crankPinRadius.value);
		g2D.setColor(crankAxleRadius.colour);
		drawCircle(g2D, tX, tY, s, crankAxleX, 0, crankAxleRadius.value);
		g2D.setColor(crankPlateRadius.colour);
		drawCircle(g2D, tX, tY, s, crankAxleX, 0, crankPlateRadius.value);

		// Piston
		g2D.setColor(pistonRodLength.colour);
		drawLine(g2D, tX, tY, s, crankPinX, crankPinY, pistonCentreX, pistonCentreY);				// Piston Rod

		g2D.setColor(pistonRadius.colour);
		drawLine(g2D, tX, tY, s,	pistonCentreX + pi1X, pistonCentreY + pi1Y,
													pistonCentreX - pi2X, pistonCentreY - pi2Y);				// Near Face
		drawLine(g2D, tX, tY, s,	pistonCentreX - pi1X, pistonCentreY - pi1Y,
													pistonCentreX + pi2X, pistonCentreY + pi2Y);				// Far Face

		// Cylinder Walls
		g2D.setColor(cylinderLength.colour);

		drawRect(g2D, tX, tY, s, cylinderAxleX, 0.0, -cylinderAngle,
			cylinderAxleX-(cylinderLength.value/2), cylinderRadius.value,
			cylinderAxleX+(cylinderLength.value/2), pistonRadius.value);

		drawRect(g2D, tX, tY, s, cylinderAxleX, 0.0, -cylinderAngle,
			cylinderAxleX-(cylinderLength.value/2), -pistonRadius.value,
			cylinderAxleX+(cylinderLength.value/2), -cylinderRadius.value);

		// Cylinder Cap
		g2D.setColor(cylinderCapRadius.colour);

		drawRect(g2D, tX, tY, s, cylinderAxleX, 0.0, -cylinderAngle,
			cylinderAxleX-(cylinderLength.value/2), cylinderCapRadius.value,
			cylinderAxleX-(cylinderLength.value/2)-cylinderCapWidth.value, -cylinderCapRadius.value);

		drawRect(g2D, tX, tY, s, cylinderAxleX, 0.0, -cylinderAngle,
			cylinderAxleX+(cylinderLength.value/2), cylinderCapRadius.value,
			cylinderAxleX+(cylinderLength.value/2)+cylinderCapWidth.value, -cylinderCapRadius.value);

		// Cylinder Cap Gland
		drawRect(g2D, tX, tY, s, cylinderAxleX, 0.0, -cylinderAngle,
			cylinderAxleX-(cylinderLength.value/2)-cylinderCapWidth.value-glandLength.value, glandRadius.value,
			cylinderAxleX-(cylinderLength.value/2)-cylinderCapWidth.value, -glandRadius.value);

		// Big End
		g2D.setColor(bigEndLength.colour);

		drawRect(g2D, tX, tY, s, crankPinX, crankPinY, -cylinderAngle,
			crankPinX + bigEndHoleOffset.value - bigEndLength.value, crankPinY - (bigEndWidth.value/2),
			crankPinX + bigEndHoleOffset.value, crankPinY + (bigEndWidth.value/2));
	}
}
