// Steam Simulator, A.Greensted
// July 2010
// http://www.labbookpages.co.uk

package steamSim.engines;

import steamSim.*;
import java.awt.*;
import javax.swing.*;

public class StevesSingleOscillator extends SteamEngine
{
	private final static Color GUIDE_COLOUR = new Color(40,40,40);

	// Dimensions all in mm
	public Parameter basePlateWidth		= new Parameter(	65.0, "Base Plate Width",
																			"Width of base plate", Color.ORANGE);
	public Parameter basePlateThickness	= new Parameter(	3.0, "Base Plate Thickness",
																			"Thickness of base plate", Color.ORANGE);

	public Parameter frameWidth			= new Parameter(	25.0, "Width of frame",
																			"Width of main central support frame", Color.ORANGE);
	public Parameter frameHeight			= new Parameter(	89.91, "Height of frame",
																			"Height of main central support frame", Color.ORANGE);

	public Parameter crankLength			= new Parameter(	8.0, "Crank Length",
																			"The distance between the crank axle and the crank pin", Color.RED);
	public Parameter crankAxlePosition	= new Parameter(	30.0, "Crank Axle Position",
																			"The distance from base of frame to crank axle centre", Color.ORANGE);
	public Parameter crankAxleRadius		= new Parameter(	3.0, "Crank Axle Radius",
																			"Radius of the crank axle", Color.ORANGE);
	public Parameter crankPinRadius		= new Parameter(	1.5, "Crank Pin Radius",
																			"Radius of the crank pin", Color.ORANGE);
	public Parameter crankPlateRadius	= new Parameter(	12.0, "Crank Plate Radius",
																			"Radius of the crank Plate", Color.ORANGE);

	public Parameter flyWheelRadius		= new Parameter(	25.0, "Fly Wheel Radius",
																			"Radius of the fly wheel, connected to crank axle", Color.YELLOW);

	public Parameter pistonLength			= new Parameter(	40.0, "Piston Length",
																			"Total piston length", Color.CYAN);
	public Parameter pistonHeadLength	= new Parameter(	12.0, "Piston Head Length",
																			"The length of the piston head", Color.CYAN);
	public Parameter pistonHeadRadius	= new Parameter(	6.0, "Piston Head Radius",
																			"The length of the piston head", Color.CYAN);
	public Parameter pistonConRodRadius	= new Parameter(	2.0, "Piston Connecting Rod Radius",
																			"Radius of the connecting rod between piston head and big end", Color.CYAN);
	public Parameter pistonBigEndLength = new Parameter(	8.0, "Piston Big End Length",
																			"The length of the piston big end", Color.CYAN);
	public Parameter pistonBigEndRadius = new Parameter(	4.0, "Piston Big End Radius",
																			"The radius of the piston big end", Color.CYAN);
	public Parameter pistonCrankOffset = new Parameter(	4.0, "Piston crank offset",
																			"Distance from big end face to centre of crank pin hole", Color.CYAN);

	public Parameter cylinderRadius		= new Parameter(	12.5, "Cylinder Radius",
																			"Radius of Cylinder", Color.PINK);
	public Parameter cylinderLength		= new Parameter(	30.0, "Cylinder Length",
																			"Length of cylinder", Color.PINK);
	public Parameter cylinderPortRadius	= new Parameter(	1.0, "Cylinder Port Radius",
																			"Radius of the cylinder port", Color.PINK);
	public Parameter cylinderPortOffset	= new Parameter(	11.0, "Cylinder Port Offset",
																			"Distance between cylinder axle and port", Color.PINK);

	public Parameter cylinderAxlePosition	= new Parameter(65.0, "Cylinder Axle Position",
																			"The distance from the base of frame to the cylinder axle centre", Color.PINK);
	public Parameter cylinderAxleRadius	= new Parameter(	2.0, "Cylinder Axle Radius",
																			"Radius of the cylinder axle", Color.PINK);

	public Parameter cylinderCapWidth	= new Parameter(	2.0, "Cylinder Cap Width",
																			"Width of cylinder cap", Color.MAGENTA);
	public Parameter cylinderCapRadius	= new Parameter(	10.0, "Cylinder Cap Radius",
																			"Radius of cylinder cap", Color.MAGENTA);

	public Parameter facePortHoleRadius	= new Parameter(	1.0, "Face Port Hole Radius",
																			"Radius of port holes in port face", Color.GREEN);
	public Parameter facePortOffset		= new Parameter(	2.5, "Face Port hole Offset",
																			"Offset from centre line to port hole", Color.GREEN);
	public Parameter facePortRadius		= new Parameter(	10.0, "Face Port Position Radius",
																			"Radius from cylinkder axle centre to port hole position", Color.GREEN);


	private double crankAngle = 0;

	public StevesSingleOscillator()
	{
		parametersArray = new Parameter[] {
			basePlateWidth, basePlateThickness,
			frameWidth, frameHeight,
			crankLength, crankAxlePosition, crankAxleRadius, crankPinRadius, crankPlateRadius,
			flyWheelRadius,
			pistonLength, pistonHeadLength, pistonHeadRadius, pistonConRodRadius, pistonBigEndLength, pistonBigEndRadius, pistonCrankOffset,
			cylinderRadius, cylinderLength, cylinderPortRadius, cylinderPortOffset,
			cylinderAxlePosition, cylinderAxleRadius,
			cylinderCapWidth, cylinderCapRadius,
			facePortHoleRadius, facePortOffset, facePortRadius,
		};

//		printTransY = 0;
//		printScale = 5.0;
	}

	public void centreView()
	{
		transX = 0;
		transY = 210;
		scale = 5.0;
	}

	public void resetAnimation()
	{
		crankAngle = 0;
	}

	public void stepAnimationBack()
	{
		crankAngle += Math.PI / 60;
		if (crankAngle > 2 * Math.PI) crankAngle -= 2 * Math.PI;
	}

	public void stepAnimationForward()
	{
		crankAngle -= Math.PI / 60;
		if (crankAngle < 0) crankAngle += 2 * Math.PI;
	}

	public void paintEngine(Graphics2D g2D, boolean printjob, int tX, int tY, double s)
	{
		double crankPinX = -crankLength.value * Math.sin(crankAngle);
		double crankPinY = -crankAxlePosition.value - crankLength.value * Math.cos(crankAngle);

		// Calculate the distance between the Cylinder Axle and the Crank Pin
		double axleSpacing = cylinderAxlePosition.value - crankAxlePosition.value;
		double distA = Math.sqrt(	axleSpacing * axleSpacing +
											crankLength.value * crankLength.value -
											2 * axleSpacing * crankLength.value * Math.cos(crankAngle));
		double cylinderAngle = Math.asin(Math.sin(crankAngle) * crankLength.value / distA);

		// Fly Wheel
		g2D.setColor(flyWheelRadius.colour);
		drawCircle(g2D, tX, tY, s, 0, -crankAxlePosition.value, flyWheelRadius.value);

		// Base Plate
		g2D.setColor(basePlateThickness.colour);
		drawRect(g2D, tX, tY, s, -basePlateWidth.value/2.0, 0, basePlateWidth.value/2.0, basePlateThickness.value);

		// Frame
		g2D.setColor(frameWidth.colour);
		drawRect(g2D, tX, tY, s, -frameWidth.value/2.0, 0, frameWidth.value/2.0, -frameHeight.value);

		// Frame Ports
		g2D.setColor(facePortRadius.colour);
		double portY = Math.sqrt(facePortRadius.value * facePortRadius.value + facePortOffset.value * facePortOffset.value);
		drawCircle(g2D, tX, tY, s, -facePortOffset.value, -(cylinderAxlePosition.value+portY), facePortHoleRadius.value);
		drawCircle(g2D, tX, tY, s, facePortOffset.value, -(cylinderAxlePosition.value+portY), facePortHoleRadius.value);

		// Crank Length
		g2D.setColor(crankLength.colour);
		drawLine(g2D, tX, tY, s, 0, -crankAxlePosition.value, crankPinX, crankPinY);

		// Axles, Crank Pin and Crank Plate
		g2D.setColor(cylinderAxleRadius.colour);
		drawCircle(g2D, tX, tY, s, 0, -cylinderAxlePosition.value, cylinderAxleRadius.value);
		g2D.setColor(crankPinRadius.colour);
		drawCircle(g2D, tX, tY, s, crankPinX, crankPinY, crankPinRadius.value);
		g2D.setColor(crankAxleRadius.colour);
		drawCircle(g2D, tX, tY, s, 0, -crankAxlePosition.value, crankAxleRadius.value);
		g2D.setColor(crankPlateRadius.colour);
		drawCircle(g2D, tX, tY, s, 0, -crankAxlePosition.value, crankPlateRadius.value);

		// Cylinder
		g2D.setColor(cylinderLength.colour);
		drawRect(g2D, tX, tY, s, 0, -cylinderAxlePosition.value, cylinderAngle,
										-cylinderRadius.value, -(cylinderAxlePosition.value+cylinderLength.value/2),
										-pistonHeadRadius.value, -cylinderAxlePosition.value+cylinderLength.value/2.0);

		g2D.setColor(cylinderLength.colour);
		drawRect(g2D, tX, tY, s, 0, -cylinderAxlePosition.value, cylinderAngle,
										cylinderRadius.value, -(cylinderAxlePosition.value+cylinderLength.value/2),
										pistonHeadRadius.value, -cylinderAxlePosition.value+cylinderLength.value/2.0);
		// Cylinder Cap
		g2D.setColor(cylinderCapWidth.colour);
		drawRect(g2D, tX, tY, s, 0, -cylinderAxlePosition.value, cylinderAngle,
										-cylinderCapRadius.value, -(cylinderAxlePosition.value+cylinderLength.value/2),
										 cylinderCapRadius.value, -(cylinderAxlePosition.value+cylinderLength.value/2.0+cylinderCapWidth.value));

		// Cylinder Port
		g2D.setColor(cylinderPortRadius.colour);
		drawCircle(g2D, tX, tY, s, 0, -cylinderAxlePosition.value, cylinderAngle,
											0, -cylinderAxlePosition.value-cylinderPortOffset.value, cylinderPortRadius.value);

		// Piston Big End
		g2D.setColor(pistonBigEndRadius.colour);
		drawRect(g2D, tX, tY, s, crankPinX, crankPinY, cylinderAngle,
			crankPinX + pistonBigEndRadius.value, crankPinY + pistonCrankOffset.value,
			crankPinX - pistonBigEndRadius.value, crankPinY + pistonCrankOffset.value - pistonBigEndLength.value);

		// Piston Head
		g2D.setColor(pistonBigEndRadius.colour);
		drawRect(g2D, tX, tY, s, crankPinX, crankPinY, cylinderAngle,
			crankPinX - pistonHeadRadius.value, crankPinY + pistonCrankOffset.value - pistonLength.value,
			crankPinX + pistonHeadRadius.value, crankPinY + pistonCrankOffset.value - pistonLength.value + pistonHeadLength.value);

		// Piston ConRod
		g2D.setColor(pistonBigEndRadius.colour);
		drawRect(g2D, tX, tY, s, crankPinX, crankPinY, cylinderAngle,
			crankPinX - pistonConRodRadius.value, crankPinY + pistonCrankOffset.value - pistonBigEndLength.value,
			crankPinX + pistonConRodRadius.value, crankPinY + pistonCrankOffset.value - pistonLength.value + pistonHeadLength.value);
	}
}
